function sroc_showall(algs, ResPth, sfb)
%
%  sroc_showall(algs, ResPth[, sfb])
%
%          plots graphs of algorithms given in algs 
%            with respect to the stereo feasibility boundary sfb
%          all ROC points are click-able showing corresponding
%            parameter setting and disparity map   
%
%  input:
%     algs ... abbreviations of algorithms to be plotted
%   ResPth ... path where results of algorithms algs are stored
%      sfb ... current feasibility boundary; if not set, default 
%              sfb from BenCOS2007 paper is taken
%
% (c) Jana Kostliva (kostliva@cmp.felk.cvut.cz)
%     CMP, FEE CTU in Prague, 23-Apr-2007

if (nargin < 3)
  load([ResPth '/sfb.mat']);
end

% tests if given sfb has the correct format
if (~correct_sfb(sfb))
   return
end

% plots the results
marks = {'h','+','o','x','*','s','d','v','^','<','>','p'};
lines = {':','-','-.','--'};
c = jet;  close
c1 = c([1:46,60:end],:);

const = 0.001; % constant for modified log-scale 
BL = 0:0.01:1; % border line

mark_size = 18;
wline = 2;
font_size = 28;
axis_size = 24;

% load results of all algorithms
Alg = [];
for al = 1:size(algs,2)
  alg = algs{al};
  load(sprintf('%s/%s/sroc_%s.mat', ResPth, alg, alg))
  % tests if sroc profile has correct format
  if (~correct_sroc(sroc))
    return
  end
  Alg{al} = sroc;
end

scenes = {'cones','slits','stripes','teddy','tsukuba',...
	  'venus','expected'};

alg_names = {algs};
for sc = 1:size(scenes,2)
  scene = scenes{sc};
  curr_sfb = getfield(sfb, sprintf('%s', scene));
  alg_names = unique([alg_names{:}, curr_sfb.algs]);
end

colours = c1(1:ceil(size(c1,1)/(size(alg_names,2))):end,:);


new_sfb = sroc_compute_sfb(algs, sprintf('%s/Results',...
					 Alg{1}.rootPth));

for sc = 1:size(scenes,2)

  scene = scenes{sc};

  % create structures for sfb alorithm plotting
  curr_sfb = getfield(sfb, sprintf('%s', scene));


  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % plots ROC points  

  figure;
  hold on;

  % plot one point from each algorithm (algs and sfb)
  % due to legend
  names = {};
  for i=1:size(alg_names,2)
    indx = find(ismember(algs, alg_names{i})); 
    if (~isempty(indx)) 
      if (~isfield(Alg{indx}, sprintf('%s', scene)))
 	continue  
      end	  
      rates = getfield(Alg{indx}, sprintf('%s',scene)); 
      plot(log(rates.SR(1)+const), log(rates.ER(1)+const),...
	   'marker', marks{mod(i,size(marks,2))+1}, 'color', colours(i,:),...
	   'linewidth', wline, 'markersize', mark_size,...
	   'linestyle','none');
      names = {names{:} alg_names{i}};
    else 
      indx = find(ismember(curr_sfb.algs,alg_names{i}));
      if (~isempty(indx))
        plot(log(curr_sfb.SR(indx(1))+const),...
	   log(curr_sfb.ER(indx(1))+const),...
	   'marker', marks{mod(i,size(marks,2))+1}, 'color',...
	   colours(i,:), 'linewidth', wline, 'markersize',...
	   mark_size, 'linestyle','none');
        names = {names{:} alg_names{i}};
      end
    end
  end

  % plot all points of all algorithms algs, make them click-able
  for al = 1:size(Alg,2)    
    if (~isfield(Alg{al}, sprintf('%s', scene)))
      continue
    end  
    rates = getfield(Alg{al}, sprintf('%s', scene));
    indx = find(ismember(alg_names,getfield(Alg{al},'alg_abbrv')));
    for i=1:size(rates.ER,2)
       handle = plot(log(rates.SR(i)+const), log(rates.ER(i)+const),...
		  'marker', marks{mod(indx, size(marks,2))+1},...
		  'color', colours(indx,:),...
		  'linewidth', wline, 'markersize', mark_size);

      str.path = Alg{al}.ResPth;
      str.alg = Alg{al}.alg_abbrv;

      string = '(';
      for j = 1:size(Alg{al}.param_name,2)-1
        string = [string Alg{al}.param_name{j} ', '];
      end
      string = [string Alg{al}.param_name{end} ') = ('];
      for j = 1:size(Alg{al}.param_name,2)-1
        string = sprintf('%s%.2f, ', string, Alg{al}.params(j,i));
      end
      string = sprintf('%s%.2f)', string, Alg{al}.params(end,i));

      str.params = string;
      str.params_num = i;

      set(handle, 'userdata', str);
      if (strcmp(scene,'expected'))
        set(handle, 'buttondownfcn','descript_fun_mean()');
        str.scene = sroc.scenes;
      else
        set(handle, 'buttondownfcn','descript_fun()');
        str.scene = scene;
      end
    end
  end

  % plot all sfb points, make them click-able
  for i=1:size(curr_sfb.ER,2)
    indx = find(ismember(alg_names,curr_sfb.algs(i)));
    handle = plot(log(curr_sfb.SR(i)+const),...
		  log(curr_sfb.ER(i)+const),...
		  'marker', marks{mod(indx,size(marks,2))+1},...
		  'color', colours(indx,:),...
		  'linewidth', wline, 'markersize', mark_size);

    str.path = [sroc.rootPth '/Results/' curr_sfb.algs{i}];
    str.alg = curr_sfb.algs{i};

    par_n = curr_sfb.param_names{i};
    pars = curr_sfb.params{i};
    string = '(';
    for j = 1:size(par_n,2)-1
      string = [string par_n{j} ', '];
    end
    string = [string par_n{end} ') = ('];
    for j = 1:size(pars,1)-1
      string = sprintf('%s%.2f, ', string, pars(j));
    end
    string = sprintf('%s%.2f)', string, pars(end));

    str.params = string;
    str.params_num = curr_sfb.param_num(i);


    set(handle, 'userdata', str);
    if (strcmp(scene,'expected'))
      set(handle, 'buttondownfcn','descript_fun_mean()');
      str.scene = sroc.scenes;
    else
      set(handle, 'buttondownfcn','descript_fun()');
      str.scene = scene; 
    end
  end

  SR = curr_sfb.SR;
  ER = curr_sfb.ER;
  boundary_sfb_sr = SR(1);
  boundary_sfb_er = ER(1);
  for bb=2:size(SR,2)
    if (ER(bb-1) > 1-SR(bb))
      boundary_sfb_sr = [boundary_sfb_sr,...
		         (1-ER(bb-1)):0.01:SR(bb), SR(bb)];
      boundary_sfb_er = [boundary_sfb_er,...
			 ER(bb-1):-0.01:(1-SR(bb)), ER(bb)];
    else
      boundary_sfb_sr = [boundary_sfb_sr SR(bb) SR(bb)];
      boundary_sfb_er = [boundary_sfb_er ER(bb-1) ER(bb)];
    end
  end
  if (boundary_sfb_sr(1) ~= 0)
    sr = boundary_sfb_sr(1);
    boundary_sfb_sr = [0:0.01:sr, sr, boundary_sfb_sr];
    boundary_sfb_er = [1:-0.01:(1-sr), (1-sr), boundary_sfb_er];
  end
  if (boundary_sfb_er(end) ~= 0)
    er = boundary_sfb_er(end);
    boundary_sfb_sr = [boundary_sfb_sr,(1-er),(1-er):0.01:1];
    boundary_sfb_er = [boundary_sfb_er, er, er:-0.01:0];
  end
  plot(log(boundary_sfb_sr+const),log(boundary_sfb_er+const),...
       'r-', 'linewidth', wline);

  plot(log(BL+const),log(BL(end:-1:1)+const),'k--');
  set(gca,'fontsize',axis_size);
  xlabel('Sparsity rate','fontsize',font_size)
  ylabel('Error rate','fontsize',font_size)
  title(sprintf('Scene: %s', scene),'fontsize',font_size)
  grid on

  legend(names,'Location','SouthWest')
  set(gcf,'name',sprintf('%s/Results/Plots/%s/ROC_%s',...
			 sroc.rootPth,scene,scene));

  ticks = [0:.01:.1 .2:.1:1];
  set(gca,'xtick',[log(ticks+const)])
  set(gca,'ytick',[log(ticks+const)])
  ticks_num = [{'0'} {'0.01'} {''} {''} {''} {''} {''} {''},...
               {''} {''} {'0.1'} {''} {''} {''} {'0.5'} {''},...
               {''} {''} {''} {'1'}]';

  set(gca,'xticklabel',ticks_num)
  set(gca,'yticklabel',ticks_num)
  xlim([log(0+const) log(1+const)])
  ylim([log(0+const) log(1+const)])


  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % plots ROC curves'

  figure;
  hold on;

  % plot one point from each algorithm (algs and sfb)
  % due to legend
  names = {};
  for i=1:size(alg_names,2)
    indx = find(ismember(algs, alg_names{i})); 
    if (~isempty(indx)) 
      if (~isfield(Alg{indx}, sprintf('%s', scene)))
 	continue  
      end	  
      rates = getfield(Alg{indx}, sprintf('%s',scene)); 
      plot(log(rates.SR(rates.ROC(1))+const),...
	   log(rates.ER(rates.ROC(1))+const),...
	   'marker', marks{mod(i,size(marks,2))+1}, 'color', colours(i,:),...
	   'linewidth', wline, 'markersize', mark_size,...
	   'linestyle',lines{mod(i,size(lines,2))+1});
      names = {names{:} alg_names{i}};
    else 
      indx = find(ismember(curr_sfb.algs,alg_names{i}));
      if (~isempty(indx)) 
        plot(log(curr_sfb.SR(indx(1))+const),...
	   log(curr_sfb.ER(indx(1))+const),...
	   'marker', marks{mod(i,size(marks,2))+1},...
	   'color', colours(i,:), 'linewidth', wline,...
	   'markersize', mark_size, 'linestyle','none');
        names = {names{:} alg_names{i}};
      end
    end
  end

  % plot ROC curve of sroc algorithm, make points click-able
  for al = 1:size(Alg,2)    
    if (~isfield(Alg{al}, sprintf('%s', scene)))
      continue
    end  
    rates = getfield(Alg{al}, sprintf('%s', scene));
    indx = find(ismember(alg_names,getfield(Alg{al},'alg_abbrv')));
    for i=1:size(rates.ROC,2)
      handle = plot(log(rates.SR(rates.ROC(i))+const),...
		    log(rates.ER(rates.ROC(i))+const),...
		    'marker', marks{mod(indx,size(marks,2))+1},...
		    'color', colours(indx,:),...
		    'linewidth', wline, 'markersize', mark_size);

      str.path = Alg{al}.ResPth;
      str.alg = Alg{al}.alg_abbrv;

      string = '(';
      for j = 1:size(Alg{al}.param_name,2)-1
        string = [string Alg{al}.param_name{j} ', '];
      end
      string = [string Alg{al}.param_name{end} ') = ('];
      for j = 1:size(Alg{al}.param_name,2)-1
        string = sprintf('%s%.2f, ', string,...
			 Alg{al}.params(j,rates.ROC(i)));
      end
      string = sprintf('%s%.2f)', string, ...
		       Alg{al}.params(end,rates.ROC(i)));

      str.params = string;
      str.params_num = rates.ROC(i);

      set(handle, 'userdata', str);
      if (strcmp(scene,'expected'))
        set(handle, 'buttondownfcn','descript_fun_mean()');
        str.scene = Alg{al}.scenes;
      else
        set(handle, 'buttondownfcn','descript_fun()');
        str.scene = scene;
      end
    end

    SR = rates.SR(rates.ROC);
    ER = rates.ER(rates.ROC);
    boundary_sr = SR(1);
    boundary_er = ER(1);
    for bb=2:size(SR,2)
      if (ER(bb-1) > 1-SR(bb))
        boundary_sr = [boundary_sr (1-ER(bb-1)):0.01:SR(bb),SR(bb)];
        boundary_er = [boundary_er, ER(bb-1):-0.01:(1-SR(bb)), ER(bb)];
      else
        boundary_sr = [boundary_sr SR(bb) SR(bb)];
        boundary_er = [boundary_er ER(bb-1) ER(bb)];
      end
    end
    if (boundary_sr(1) ~= 0)
      sr = boundary_sr(1);
      boundary_sr = [0:0.01:sr, sr, boundary_sr];
      boundary_er = [1:-0.01:(1-sr), (1-sr), boundary_er];
    end
    if (boundary_er(end) ~= 0)
      er = boundary_er(end);
      boundary_sr = [boundary_sr,(1-er),(1-er):0.01:1];
      boundary_er = [boundary_er, er, er:-0.01:0];
    end
    plot(log(boundary_sr+const),log(boundary_er+const),...
	 'color', colours(indx,:), 'linestyle',...
	 lines{mod(indx,size(lines,2))+1},...
	 'linewidth', wline, 'markersize', mark_size);
  end	
    

  % plot all sfb points, make them click-able
  for i=1:size(curr_sfb.ER,2)
    indx = find(ismember(alg_names,curr_sfb.algs(i)));
    handle = plot(log(curr_sfb.SR(i)+const),...
		  log(curr_sfb.ER(i)+const),...
		  'marker', marks{mod(indx,size(marks,2))+1},...
		  'color', colours(indx,:),...
		  'linewidth', wline, 'markersize', mark_size);

    str.path = [sroc.rootPth '/Results/' curr_sfb.algs{i}];
    str.alg = curr_sfb.algs{i};

    par_n = curr_sfb.param_names{i};
    pars = curr_sfb.params{i};
    string = '(';
    for j = 1:size(par_n,2)-1
      string = [string par_n{j} ', '];
    end
    string = [string par_n{end} ') = ('];
    for j = 1:size(pars,1)-1
      string = sprintf('%s%.2f, ', string, pars(j));
    end
    string = sprintf('%s%.2f)', string, pars(end));

    str.params = string;
    str.params_num = curr_sfb.param_num(i);

    set(handle, 'userdata', str);
    if (strcmp(scene,'expected'))
      set(handle, 'buttondownfcn','descript_fun_mean()');
      str.scene = sroc.scenes;
    else
      set(handle, 'buttondownfcn','descript_fun()');
      str.scene = scene; 
    end
  end

  % plot sfb curve
  plot(log(boundary_sfb_sr+const),log(boundary_sfb_er+const),...
       'r-', 'linewidth', wline);

  plot(log(BL+const),log(BL(end:-1:1)+const),'k--');
  set(gca,'fontsize',axis_size);
  xlabel('Sparsity rate','fontsize',font_size)
  ylabel('Error rate','fontsize',font_size)
  title(sprintf('ROC curves - Scene: %s', scene),...
	'fontsize',font_size)
  grid on

  legend(names,'Location','SouthWest')
  set(gcf,'name',sprintf('%s/Results/Plots/%s/ROC_curves_%s',...
			 sroc.rootPth,scene,scene));

  ticks = [0:.01:.1 .2:.1:1];
  set(gca,'xtick',[log(ticks+const)])
  set(gca,'ytick',[log(ticks+const)])
  ticks_num = [{'0'} {'0.01'} {''} {''} {''} {''} {''} {''},...
               {''} {''} {'0.1'} {''} {''} {''} {'0.5'} {''},...
               {''} {''} {''} {'1'}]';

  set(gca,'xticklabel',ticks_num)
  set(gca,'yticklabel',ticks_num)
  xlim([log(0+const) log(1+const)])
  ylim([log(0+const) log(1+const)])

 %end

 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 % plot SFB from algorithms algs and current_sfb

  figure;
  hold on;

  new_sfb_sc = getfield(new_sfb,sprintf('%s', scene));

  names = {};
  % plot one point from each algorithm (new_sfb and sfb)
  % due to legend
  for i=1:size(alg_names,2)
    indx = find(ismember(new_sfb_sc.algs, alg_names{i})); 
    if (~isempty(indx)) 
      plot(log(new_sfb_sc.SR(indx(1))+const),...
	   log(new_sfb_sc.ER(indx(1))+const),...
	   'marker', marks{mod(i,size(marks,2))+1}, 'color',...
           colours(i,:), 'linewidth', wline, 'markersize',...
           mark_size,'linestyle','none');
      names = {names{:} alg_names{i}};
    else 
      indx = find(ismember(curr_sfb.algs,alg_names{i}));
      if (~isempty(indx)) 
        plot(log(curr_sfb.SR(indx(1))+const),...
	   log(curr_sfb.ER(indx(1))+const),...
	   'marker', marks{mod(i,size(marks,2))+1},...
	   'color', colours(i,:), 'linewidth', wline,...
	   'markersize', mark_size, 'linestyle','none');
        names = {names{:} alg_names{i}};
      end
    end
  end

  % plot all new_sfb points, make them click-able
  for i=1:size(new_sfb_sc.ER,2)
    indx = find(ismember(alg_names,new_sfb_sc.algs(i)));
    handle = plot(log(new_sfb_sc.SR(i)+const),...
		  log(new_sfb_sc.ER(i)+const),...
		  'marker', marks{mod(indx,size(marks,2))+1},...
		  'color', colours(indx,:),...
		  'linewidth', wline, 'markersize', mark_size);

    str.path = [ResPth '/' new_sfb_sc.algs{i}];
    str.alg = new_sfb_sc.algs{i};

    par_n = new_sfb_sc.param_names{i};
    pars = new_sfb_sc.params{i};
    string = '(';
    for j = 1:size(par_n,2)-1
      string = [string par_n{j} ', '];
    end
    string = [string par_n{end} ') = ('];
    for j = 1:size(pars,1)-1
      string = sprintf('%s%.2f, ', string, pars(j));
    end
    string = sprintf('%s%.2f)', string, pars(end));

    str.params = string;
    str.params_num = new_sfb_sc.param_num(i);

    set(handle, 'userdata', str);
    if (strcmp(scene,'expected'))
      set(handle, 'buttondownfcn','descript_fun_mean()');
      str.scene = {scenes{1:end-1}};
    else
      set(handle, 'buttondownfcn','descript_fun()');
      str.scene = scene; 
    end
  end

  SR = new_sfb_sc.SR;
  ER = new_sfb_sc.ER;
  boundary_new_sfb_sr = SR(1);
  boundary_new_sfb_er = ER(1);
  for bb=2:size(SR,2)
    if (ER(bb-1) > 1-SR(bb))
      boundary_new_sfb_sr = [boundary_new_sfb_sr,...
     		             (1-ER(bb-1)):0.01:SR(bb), SR(bb)];
      boundary_new_sfb_er = [boundary_new_sfb_er,...
			     ER(bb-1):-0.01:(1-SR(bb)), ER(bb)];
    else
      boundary_new_sfb_sr = [boundary_new_sfb_sr SR(bb) SR(bb)];
      boundary_new_sfb_er = [boundary_new_sfb_er ER(bb-1) ER(bb)];
    end
  end
  if (boundary_new_sfb_sr(1) ~= 0)
    sr = boundary_new_sfb_sr(1);
    boundary_new_sfb_sr = [0:0.01:sr, sr, boundary_new_sfb_sr];
    boundary_new_sfb_er = [1:-0.01:(1-sr), (1-sr), boundary_new_sfb_er];
  end
  if (boundary_new_sfb_er(end) ~= 0)
    er = boundary_new_sfb_er(end);
    boundary_new_sfb_sr = [boundary_new_sfb_sr,(1-er),(1-er):0.01:1];
    boundary_new_sfb_er = [boundary_new_sfb_er, er, er:-0.01:0];
  end

  % plot new_sfb curve
  plot(log(boundary_new_sfb_sr+const),log(boundary_new_sfb_er+const),...
       '-', 'color',colours(1,:),'linewidth', wline);

  % plot all sfb points, make them click-able
  for i=1:size(curr_sfb.ER,2)
    indx = find(ismember(alg_names,curr_sfb.algs(i)));
    handle = plot(log(curr_sfb.SR(i)+const),...
		  log(curr_sfb.ER(i)+const),...
		  'marker', marks{mod(indx,size(marks,2))+1},...
		  'color', colours(indx,:),...
		  'linewidth', wline, 'markersize', mark_size);

    str.path = [ResPth '/' curr_sfb.algs{i}];
    str.alg = curr_sfb.algs{i};

    par_n = curr_sfb.param_names{i};
    pars = curr_sfb.params{i};
    string = '(';
    for j = 1:size(par_n,2)-1
      string = [string par_n{j} ', '];
    end
    string = [string par_n{end} ') = ('];
    for j = 1:size(pars,1)-1
      string = sprintf('%s%.2f, ', string, pars(j));
    end
    string = sprintf('%s%.2f)', string, pars(end));

    str.params = string;
    str.params_num = curr_sfb.param_num(i);

    set(handle, 'userdata', str);
    if (strcmp(scene,'expected'))
      set(handle, 'buttondownfcn','descript_fun_mean()');
      str.scene = sroc.scenes;
    else
      set(handle, 'buttondownfcn','descript_fun()');
      str.scene = scene; 
    end
  end

  % plot sfb curve
  plot(log(boundary_sfb_sr+const),log(boundary_sfb_er+const),...
       'r-', 'linewidth', wline);


  plot(log(BL+const),log(BL(end:-1:1)+const),'k--');
  set(gca,'fontsize',axis_size);
  xlabel('Sparsity rate','fontsize',font_size)
  ylabel('Error rate','fontsize',font_size)
  title(sprintf('SFB - Scene: %s', scene),...
	'fontsize',font_size)
  grid on

  legend(names,'Location','SouthWest')
  set(gcf,'name',sprintf('%s/Results/Plots/%s/ROC_sfb_%s',...
			 sroc.rootPth,scene,scene));

  ticks = [0:.01:.1 .2:.1:1];
  set(gca,'xtick',[log(ticks+const)])
  set(gca,'ytick',[log(ticks+const)])
  ticks_num = [{'0'} {'0.01'} {''} {''} {''} {''} {''} {''},...
               {''} {''} {'0.1'} {''} {''} {''} {'0.5'} {''},...
               {''} {''} {''} {'1'}]';

  set(gca,'xticklabel',ticks_num)
  set(gca,'yticklabel',ticks_num)
  xlim([log(0+const) log(1+const)])
  ylim([log(0+const) log(1+const)])

end

exp = 0;
for al = 1:size(Alg,2)
  if (Alg{al}.export_results)
    exp = 1;
    break;
  end
end

if (exp)
  tofiles eps
  tofiles fig
end
